/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of the edge detection algorithm implemented with Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiFilterSobelNegVertBorder_8u16s_C1R
//     ippiFilterSobelHorizBorder_8u16s_C1R
//     ippiCanny_16s8u_C1R


#include <stdio.h>
#include "ipp.h"

#define WIDTH  128  /* image width */
#define HEIGHT  64  /* image height */
#define THRESH_LOW    50.f /* Low   threshold for edges detection */
#define THRESH_HIGHT 150.f /* Upper threshold for edges detection */
#define BORDER_VAL 0

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    IppStatus status = ippStsNoErr;
    Ipp8u* pSrc = NULL, *pDst = NULL;  /* Pointers to source/destination images */
    int srcStep = 0, dstStep = 0;      /* Steps, in bytes, through the source/destination images */
    IppiSize roiSize = { WIDTH, HEIGHT }; /* Size of source/destination ROI in pixels */

    int iTmpBufSizeSobV  = 0;
    int iTmpBufSizeSobH  = 0;
    int iTmpBufSizeCanny = 0;
    int iTmpBufSize = 0;         /* Common work buffer size */
    int  dxStep = 0, dyStep = 0; /* Steps, in bytes, through buffer_dx/buffer_dy */
    Ipp8u *buffer = NULL;        /* Pointer to the common work buffer */
    Ipp16s* buffer_dx = NULL, *buffer_dy = NULL; /* Pointer to the buffer for first derivatives with respect to X / Y */

    pSrc = ippiMalloc_8u_C1(roiSize.width, roiSize.height, &srcStep);
    pDst = ippiMalloc_8u_C1(roiSize.width, roiSize.height, &dstStep);

    /* Start Edge Detection algorithm */
    {
        buffer_dx = ippiMalloc_16s_C1(roiSize.width, roiSize.height, &dxStep);
        buffer_dy = ippiMalloc_16s_C1(roiSize.width, roiSize.height, &dyStep);

        check_sts(  status = ippiFilterSobelVertBorderGetBufferSize(roiSize, ippMskSize3x3, ipp8u, ipp16s, 1, &iTmpBufSizeSobV) )
        check_sts(  status = ippiFilterSobelHorizBorderGetBufferSize(roiSize, ippMskSize3x3, ipp8u, ipp16s, 1, &iTmpBufSizeSobH) )

        check_sts(  ippiCannyGetSize(roiSize, &iTmpBufSizeCanny) )

        /* Find maximum buffer size */
        iTmpBufSize = (iTmpBufSizeSobV > iTmpBufSizeSobH) ? iTmpBufSizeSobV : iTmpBufSizeSobH;
        iTmpBufSize = (iTmpBufSize > iTmpBufSizeCanny) ? iTmpBufSize : iTmpBufSizeCanny;
        buffer = ippsMalloc_8u(iTmpBufSize);

        check_sts(  status = ippiFilterSobelNegVertBorder_8u16s_C1R(pSrc, srcStep, buffer_dx, dxStep, roiSize, ippMskSize3x3, ippBorderRepl, BORDER_VAL, buffer) )

        check_sts(  status = ippiFilterSobelHorizBorder_8u16s_C1R(pSrc, srcStep, buffer_dy, dyStep, roiSize, ippMskSize3x3, ippBorderRepl, BORDER_VAL, buffer) )

        check_sts(  status = ippiCanny_16s8u_C1R(buffer_dx, dxStep, buffer_dy, dyStep, pDst, dstStep, roiSize, THRESH_LOW, THRESH_HIGHT, buffer) )
    }

EXIT_MAIN
    ippsFree(buffer);
    ippiFree(buffer_dx);
    ippiFree(buffer_dy);
    ippiFree(pSrc);
    ippiFree(pDst);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
