/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of gradient vectors computation over an image with the Sobel
// operator using Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiGradientVectorGetBufferSize
//     ippiGradientVectorSobel_8u16s_C3C1R


#include <stdio.h>
#include "ipp.h"

#define WIDTH   8     /* source image width */
#define HEIGHT  4     /* source image height */
#define ROI_WIDTH  8  /* source image ROI width */
#define ROI_HEIGHT 4  /* source image ROI height */
#define NUMCHN  1     /* number of channels */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

static Ipp8u pSrc[(NUMCHN * WIDTH + 1)* HEIGHT] = /* Pointer to source image */
{ 1, 2, 3, 4, 8, 8, 8, 8,
  1, 2, 3, 4, 8, 8, 8, 8,
  1, 2, 3, 4, 8, 8, 8, 8,
  1, 2, 3, 4, 8, 8, 8, 8 };

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status */

int main(void)
{
    IppStatus status = ippStsNoErr;
    int srcStep = WIDTH*NUMCHN;                   /* Steps, in bytes, through the source images */
    Ipp8u* psrc = pSrc + 1*srcStep + 1 * NUMCHN;   /* Pointer to source ROI image */
    IppiSize roiSize = { ROI_WIDTH, ROI_HEIGHT }; /* Size of source ROI in pixels */
    Ipp16s pGx[4 * 7], pGy[4 * 7]; /* Define x and y gradient components */
    int gxStep = 4 * sizeof(Ipp16s), gyStep = 4 * sizeof(Ipp16s);/* Steps through the X and Y component image */
    Ipp8u* pBuffer = NULL;                   /* Pointer to the work buffer */
    Ipp8u borderValue = 0;
    int bufferSize = 0;
    Ipp16s pMag[ROI_WIDTH*ROI_HEIGHT];       /* Pointer to the magnitude of computed gradient */
    int magStep = ROI_WIDTH*sizeof(Ipp16s);  /* Step through the magnitude image */
    Ipp32f pAngle[ROI_WIDTH*ROI_HEIGHT];     /* Pointer to the angle of computed gradient */
    int angleStep = ROI_WIDTH*sizeof(Ipp32f);/* Step through the magnitude image */
    IppiMaskSize maskSize = ippMskSize3x3;
    IppiBorderType borderType = ippBorderConst;

    check_sts( status = ippiGradientVectorGetBufferSize(roiSize, maskSize, ipp8u, NUMCHN, &bufferSize) )

    pBuffer = ippsMalloc_8u(bufferSize);

    /* Compute gradient using 3x3 Sobel operator (source ROI at point x=1, y=1) */
    check_sts( status = ippiGradientVectorSobel_8u16s_C1R(psrc, srcStep, pGx, gxStep, pGy, gyStep,
        pMag, magStep, pAngle, angleStep, roiSize, maskSize, ippNormL2, borderType, borderValue, pBuffer) )

EXIT_MAIN
    ippsFree(pBuffer);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
