/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of detecting lines in the source image using
// the probabilistic Hough transform using Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiHoughProbLineGetSize_8u_C1R
//     ippiHoughProbLineInit_8u32f_C1R
//     ippiHoughProbLine_8u32f_C1R


#include <stdio.h>
#include "ipp.h"

#define WIDTH  800  /* image width  */
#define HEIGHT 600  /* image height */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    int i = 0;
    IppStatus status = ippStsNoErr;
    Ipp8u* pSrc = NULL;    /* Pointers to source images */
    int srcStep = 0;       /* Steps, in bytes, through the source images */
    IppiSize roiSize = { WIDTH, HEIGHT };    /* Size of source ROI in pixels */
    int bufferSize = 0, specSize = 0;/* Size of the working buffer and spec structure */
    Ipp8u *pBuffer = NULL;
    IppiHoughProbSpec*  pSpec = NULL;
    IppiPoint pLineP[128]; /* Pointer to output array of detected lines */
    int threshold = 20, lineLength = 40, ineGap = 4, maxLineCount = 128;
    IppPointPolar delta = { 1, (Ipp32f)IPP_PI / 180.f / 3.f }; /* Step of discretization */
    int  lineCount = 0; /* Number of detected lines. */

    pSrc = ippiMalloc_8u_C1(roiSize.width, roiSize.height, &srcStep);

    check_sts( status = ippiSet_8u_C1R(0, pSrc, srcStep, roiSize) ) /* Zero image */

    for (i = 0; i<70; i++) {     /* Draw line (100,100)-(170,170) */
        pSrc[(i + 100)*srcStep + i + 100] = 255;
    }

    /* Compute the size of the working buffer and spec structure */
    check_sts( status = ippiHoughProbLineGetSize_8u_C1R(roiSize, delta, &specSize, &bufferSize) )

    pBuffer = (Ipp8u*)ippsMalloc_8u(bufferSize);
    pSpec = (IppiHoughProbSpec*)ippsMalloc_8u(specSize);

    /* Initialize the specification structure */
    check_sts( status = ippiHoughProbLineInit_8u32f_C1R(roiSize, delta, ippAlgHintNone, pSpec) )

    /* Detect lines in the source image using the probabilistic Hough transform. */
    check_sts( status = ippiHoughProbLine_8u32f_C1R(pSrc, srcStep, roiSize, threshold, lineLength, ineGap, pLineP, maxLineCount, &lineCount, pBuffer, pSpec) )

EXIT_MAIN
    ippsFree(pBuffer);
    ippiFree(pSrc);
    ippsFree(pSpec);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
