/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of the warp perspective transform 
// using Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiWarpPerspectiveGetSize
//     ippiWarpPerspectiveCubicInit
//     ippiWarpGetBufferSize
//     ippiWarpPerspectiveCubic_8u_C3R


#include <stdio.h>
#include "ipp.h"

#define WIDTH  640 /* Source image width */
#define HEIGHT 480 /* Destination image width */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

static const double coeffs[3][3] = { /* Coefficients for the perspective transform */
    { 2.3439242437790240     , 0.0025160437043002296  , -499.09988916944815, },
    { 0.0058031616363677600  , 1.4997057893818542     , -181.04366815436885, },
    { 1.1527998857985334e-005, 1.9607666631981373e-006, 0.99730924424366529  }
};

/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    IppStatus status = ippStsNoErr;
    Ipp8u* pSrc = NULL, *pDst = NULL; /* Pointers to source/destination images */
    int srcStep = 0, dstStep = 0;     /* Steps, in bytes, through the source/destination images */
    IppiSize srcSize = { WIDTH, HEIGHT }, dstSize = { WIDTH, HEIGHT }; /* Size of source/destination ROI in pixels */
    IppiWarpSpec* pSpec = NULL; /* Pointer to the specification structure */
    Ipp8u* pInitBuf = NULL;     /* Pointer to temporary buffer for transform initialization*/
    Ipp8u* pBuffer  = NULL;     /* Pointer to the work buffer */

    pSrc = ippiMalloc_8u_C3(srcSize.width, srcSize.height, &srcStep);
    pDst = ippiMalloc_8u_C3(dstSize.width, dstSize.height, &dstStep);

    /* Start perspective transform */
    {
        int    specSize = 0;        /* Size, in bytes, of the specification structure */
        int    initSize = 0;        /* Size, in bytes, of the temporary buffer */
        int    bufSize  = 0;        /* Size, in bytes, of the work buffer */
        const Ipp32s numChannels = 3;
        IppiPoint dstOffset = { 0, 0 }; /* Offset of the destination image ROI with respect to the destination image origin */
        IppStatus status = ippStsNoErr;
        IppiBorderType   borderType = ippBorderConst;
        IppiWarpDirection direction = ippWarpForward; /* Transformation direction */
        Ipp64f pBorderValue[3];
        Ipp64f valB = 0.0, valC = 0.5;     /* Catmull-Rom filter coefficients for cubic interpolation */
        IppiRect srcRoi = ippRectInfinite; /* Region of interest is not specified */
        int smoothEdge = 0; /* Flag for edge smoothing */
        IppiInterpolationType interpolation = ippCubic;
        int i = 0;

        for (i = 0; i < numChannels; ++i) pBorderValue[i] = 255.0;

        /* Computes the size of the specification structure and the size of the external work buffer for the warp perspective transform */
        check_sts( status = ippiWarpPerspectiveGetSize(srcSize, srcRoi, dstSize, ipp8u, coeffs, interpolation, direction, borderType, &specSize, &initSize) )

        pSpec    = (IppiWarpSpec*)ippsMalloc_8u(specSize);
        pInitBuf = ippsMalloc_8u(initSize);

        /* Initializes the specification structure for image perspective warping with the cubic interpolation method */
        check_sts( status = ippiWarpPerspectiveCubicInit(srcSize, srcRoi, dstSize, ipp8u, coeffs, direction, numChannels, valB, valC, borderType, pBorderValue, smoothEdge, pSpec, pInitBuf) )

        /* Work buffer size */
        check_sts( status = ippiWarpGetBufferSize(pSpec, dstSize, &bufSize) )

        pBuffer = ippsMalloc_8u(bufSize);

        /* Performs warp perspective transformation of an image using the cubic interpolation method */
        check_sts( status = ippiWarpPerspectiveCubic_8u_C3R(pSrc, srcStep, pDst, dstStep, dstOffset, dstSize, pSpec, pBuffer) )
    }

EXIT_MAIN
    ippsFree(pInitBuf);
    ippsFree(pSpec);
    ippsFree(pBuffer);
    ippiFree(pSrc);
    ippiFree(pDst);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
