/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of the Spec structure initialization for warping of
// an arbitrary quadrangle in the source image to the quadrangle
// in the destination image using Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiWarpQuadGetSize
//     ippiWarpQuadLinearInit
//     ippiWarpGetBufferSize
//     ippiWarpPerspectiveLinear_8u_C3R


#include <stdio.h>
#include "ipp.h"

#define WIDTH  640 /* Source image width */
#define HEIGHT 480 /* Destination image width */

/* Next two defines are created to simplify code reading and understanding */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

static const double srcQuad[4][2] = /* Vertex coordinates of the quadrangle */
{
    { 0.0, 0.0 },
    { 8.0, 0.0 },
    { 6.0, 8.0 },
    { 2.0, 8.0 } };

static const double dstQuad[4][2] = /* Vertex coordinates of the quadrangle */
{
    { 0.0, 0.0 },
    { 8.0, 0.0 },
    { 6.0, 8.0 },
    { 2.0, 8.0 } };


/* Results of ippMalloc() are not validated because Intel(R) IPP functions perform bad arguments check and will return an appropriate status  */

int main(void)
{
    IppStatus status = ippStsNoErr;
    Ipp8u* pSrc = NULL, *pDst = NULL;  /* Pointers to source/destination images */
    int srcStep = 0, dstStep = 0;      /* Steps, in bytes, through the source/destination images */
    IppiSize srcSize = { WIDTH, HEIGHT }, dstSize = { WIDTH, HEIGHT }; /* Size of source/destination ROI in pixels */
    IppiInterpolationType interpolation = ippLinear;
    IppiBorderType borderType = ippBorderTransp; /* Transparent border : destination image pixels mapped to the outer source image pixels are not changed. */
    int    specSize = 0, bufSize = 0;            /* Size, in bytes, of the specification/temporary buffer structure */
    IppiWarpTransformType warpTransformType = ippWarpPerspective;
    IppiWarpSpec* pSpec = NULL; /* Pointer to the specification structure/work buffer */
    Ipp8u* pBuffer = NULL;
    IppiPoint dstOffset = { 0, 0 }; /* Offset of the destination image ROI with respect to the destination image origin */
    int numChannels = 3;

    pSrc = ippiMalloc_8u_C3(srcSize.width, srcSize.height, &srcStep);
    pDst = ippiMalloc_8u_C3(dstSize.width, dstSize.height, &dstStep);

    /* Spec and init buffer sizes */
    check_sts( status = ippiWarpQuadGetSize(srcSize, srcQuad, dstSize, dstQuad, warpTransformType, ipp8u, interpolation, borderType, &specSize, &bufSize) )

    pSpec = (IppiWarpSpec*)ippsMalloc_8u(specSize);

    /* Filter initialization */
    check_sts( status = ippiWarpQuadLinearInit(srcSize, srcQuad, dstSize, dstQuad, warpTransformType, ipp8u, numChannels, borderType, 0, 0, pSpec) )

    check_sts( status = ippiWarpGetBufferSize(pSpec, dstSize, &bufSize) )

    pBuffer = ippsMalloc_8u(bufSize);
    /* Perform perspective transform of an image */
    check_sts( status = ippiWarpPerspectiveLinear_8u_C3R(pSrc, srcStep, pDst, dstStep, dstOffset, dstSize, pSpec, pBuffer) )

EXIT_MAIN
    ippsFree(pSpec);
    ippsFree(pBuffer);
    ippiFree(pSrc);
    ippiFree(pDst);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}
