/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of forward/inverse wavelet Daubechies 2 (db2) transform
// implemented with Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippsWTFwdGetSize
//     ippsWTFwdInit_32f
//     ippsWTFwdSetDlyLine_32f
//     ippsWTFwd_32f
//     ippsWTInvGetSize
//     ippsWTInvInit_32f
//     ippsWTInvSetDlyLine_32f

#include <stdio.h> 
#include "ipp.h"
//   Filter bank for Daubechies, order 2
#define FILTER_LENGTH 4

static const int fwdFltLenL = FILTER_LENGTH;
static const int fwdFltLenH = FILTER_LENGTH;

static const float fwdFltL[FILTER_LENGTH] =
{/* Pointer to lowpass filter taps */
    -1.294095225509215e-001f,
    2.241438680418574e-001f,
    8.365163037374690e-001f,
    4.829629131446903e-001f
};

static const float fwdFltH[FILTER_LENGTH] =
{/* Pointer to highpass filter taps */
    -4.829629131446903e-001f,
    8.365163037374690e-001f,
    -2.241438680418574e-001f,
    -1.294095225509215e-001f
};

static const int invFltLenL = FILTER_LENGTH;
static const int invFltLenH = FILTER_LENGTH;
static const float invFltL[FILTER_LENGTH] =
{/* Pointer to lowpass filter taps */
    4.829629131446903e-001f,
    8.365163037374690e-001f,
    2.241438680418574e-001f,
    -1.294095225509215e-001f
};

static const float invFltH[FILTER_LENGTH] =
{/* Pointer to highpass filter taps */
    -1.294095225509215e-001f,
    -2.241438680418574e-001f,
    8.365163037374690e-001f,
    -4.829629131446903e-001f
};

/* minimal values */
static const int fwdFltOffsL = -1; /* Input delay of lowpass filter  */
static const int fwdFltOffsH = -1; /* Input delay of highpass filter */

/* minimal values, that corresponds to perfect reconstruction */
static const int invFltOffsL = 0; /* Input delay of lowpass filter  */
static const int invFltOffsH = 0; /* Input delay of highpass filter */
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

int main(void)
{
    int i;
    IppStatus status = ippStsNoErr;
    IppsWTFwdState_32f* pStateFwd = 0; /* Pointer to allocated ippsWTFwd state structure */
    IppsWTInvState_32f* pStateInv = 0; /* Pointer to allocated ippsWTInv state structure */
    int pStateSize;
    IppDataType srcType = ipp32f; /* Data type of the source vector */
    Ipp32f src[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 };/* pointer to source block of data */
    Ipp32f dst[12]; /* pointer to destination block of reconstructed data */
    Ipp32f low[7];  /* pointer to destination block of "low-frequency"  component */
    Ipp32f high[7]; /* pointer to destination block of "high-frequency" component */

    printf("original:\n");
    for (i = 0; i < 12; i++) printf("%.0f; ", src[i]);
    printf("\n");
    check_sts(status = ippsWTFwdGetSize(srcType, fwdFltLenL, fwdFltOffsL, fwdFltLenH, fwdFltOffsH, &pStateSize));
    pStateFwd = (IppsWTFwdState_32f*)ippsMalloc_8u(pStateSize);
    check_sts(status = ippsWTFwdInit_32f(pStateFwd, fwdFltL, fwdFltLenL, fwdFltOffsL, fwdFltH, fwdFltLenH, fwdFltOffsH));
    /* We substitute wrapping extension in "the beginning of stream"
    Here should be the same pointers for this offsets, but in the general case it may be different */
    check_sts(status = ippsWTFwdSetDlyLine_32f(pStateFwd, &src[10], &src[10]));
    /* Forward transform */
    check_sts(status = ippsWTFwd_32f(src, low, high, 6, pStateFwd));
    /* print decomposition result */
    printf("approx:\n");
    for (i = 0; i < 6; i++) printf("%.4f; ", low[i]);
    printf("\n");
    printf("details:\n");
    for (i = 0; i < 6; i++) printf("%.4f; ", high[i]);
    printf("\n");

    check_sts(status = ippsWTInvGetSize(srcType, invFltLenL, invFltOffsL, invFltLenH, invFltOffsH, &pStateSize));
    pStateInv = (IppsWTInvState_32f*)ippsMalloc_8u(pStateSize);
    check_sts(status = ippsWTInvInit_32f(pStateInv, invFltL, invFltLenL, invFltOffsL, invFltH, invFltLenH, invFltOffsH));
    /* For this particular case (non-shifted reconstruction)
     here is first data itself,
     that we need to place to delay line
     [(invFltLenL + invFltOffsL - 1) / 2] elements for l. filtering
     [(invFltLenH + invFltOffsH - 1) / 2] elements for h. filtering
    */
    check_sts(status = ippsWTInvSetDlyLine_32f(pStateInv, low, high));

    /* Inverse transform */
    check_sts(status = ippsWTInv_32f(&low[1], &high[1], 5, dst, pStateInv));
    /* Here are the substitution of the wrapping extension 
    at the "end of stream" and calculation of last samples of reconstruction
    We do not use additional buffer and do not copy any data externally,
    just substitute beginning of input data itself to simulate wrapping
    */
    check_sts(status = ippsWTInv_32f(low, high, 1, &dst[10], pStateInv));
    /* print reconstruction result */
    printf("reconstruction:\n");
    for (i = 0; i < 12; i++) printf("%.0f; ", dst[i]);
    printf("\n");

    EXIT_MAIN
    ippsFree(pStateFwd);
    ippsFree(pStateInv);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}