/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of forward/inverse wavelet Daubechies 3 (db3) transform
// implemented with Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippsWTFwdGetSize
//     ippsWTFwdInit_32f
//     ippsWTFwdSetDlyLine_32f
//     ippsWTFwd_32f
//     ippsWTInvGetSize
//     ippsWTInvInit_32f
//     ippsWTInvSetDlyLine_32f

#include <stdio.h> 
#include "ipp.h"
//   Filter bank for Daubechies 3 (db3)
#define FILTER_LENGTH 6

static const int fwdFltLenL = FILTER_LENGTH;
static const int fwdFltLenH = FILTER_LENGTH;

static const float fwdFltL[FILTER_LENGTH] =
{/* Pointer to lowpass filter taps */
    0.035226291882100656f,
    -0.08544127388224149f,
    -0.13501102001039084f,
    0.4598775021193313f,
    0.8068915093133388f,
    0.3326705529509569f

};

static const float fwdFltH[FILTER_LENGTH] =
{/* Pointer to highpass filter taps */
    -0.3326705529509569f,
    0.8068915093133388f,
    -0.4598775021193313f,
    -0.13501102001039084f,
    0.08544127388224149f,
    0.035226291882100656f

};

static const int invFltLenL = FILTER_LENGTH;
static const int invFltLenH = FILTER_LENGTH;

static const float invFltL[FILTER_LENGTH] =
{/* Pointer to lowpass filter taps */
    0.3326705529509569f,
    0.8068915093133388f,
    0.4598775021193313f,
    -0.13501102001039084f,
    -0.08544127388224149f,
    0.035226291882100656f,
};

static const float invFltH[FILTER_LENGTH] =
{/* Pointer to highpass filter taps */
    0.035226291882100656f,
    0.08544127388224149f,
    -0.13501102001039084f,
    -0.4598775021193313f,
    0.8068915093133388f,
    -0.3326705529509569f
};

// minimal values 
static const int fwdFltOffsL = -1;
static const int fwdFltOffsH = -1;

// minimal values, that corresponds to perfect reconstruction 
static const int invFltOffsL = 0;
static const int invFltOffsH = 0;
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

int main(void)
{
    int i;
    IppStatus status = ippStsNoErr;
    IppsWTFwdState_32f* pStateFwd = 0; /* Pointer to allocated ippsWTFwd state structure */
    IppsWTInvState_32f* pStateInv = 0; /* Pointer to allocated ippsWTInv state structure */
    int pStateSize;
    IppDataType srcType = ipp32f; /* Data type of the source vector */
    Ipp32f src[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 };/* pointer to source block of data */
    Ipp32f dst[12]; /* pointer to destination block of reconstructed data */
    Ipp32f low[7];  /* pointer to destination block of "low-frequency"  component */
    Ipp32f high[7]; /* pointer to destination block of "high-frequency" component */
    Ipp32f dLineFwdL[4] = { src[8], src[9], src[10], src[11] };   // wrap [fwdFltLenL + fwdFltOffsL - 1]
    Ipp32f dLineFwdH[4] = { src[8], src[9], src[10], src[11] };   // wrap [fwdFltLenL + fwdFltOffsL - 1]
    Ipp32f dLineInvL[2], dLineInvH[2];

    printf("original:\n");
    for (i = 0; i < 12; i++) printf("%.0f; ", src[i]);
    printf("\n");
    check_sts(status = ippsWTFwdGetSize(srcType, fwdFltLenL, fwdFltOffsL, fwdFltLenH, fwdFltOffsH, &pStateSize));
    pStateFwd = (IppsWTFwdState_32f*)ippsMalloc_8u(pStateSize);
    check_sts(status = ippsWTFwdInit_32f(pStateFwd, fwdFltL, fwdFltLenL, fwdFltOffsL, fwdFltH, fwdFltLenH, fwdFltOffsH));
    /* We substitute wrapping extension in "the beginning of stream"
    Here should be the same pointers for this offsets, but in the general case it may be different */
    check_sts(status = ippsWTFwdSetDlyLine_32f(pStateFwd, dLineFwdL, dLineFwdH));    // wrap [fwdFltLenL + fwdFltOffsL - 1]
    /* Forward transform */
    check_sts(status = ippsWTFwd_32f(src, low, high, 6, pStateFwd));
    /* print decomposition result */
    printf("approx:\n");
    for (i = 0; i < 6; i++) printf("%.4f; ", low[i]);
    printf("\ndetails:\n");
    for (i = 0; i < 6; i++) printf("%.4f; ", high[i]);
    printf("\n");
    check_sts(status = ippsWTInvGetSize(srcType, invFltLenL, invFltOffsL, invFltLenH, invFltOffsH, &pStateSize));
    pStateInv = (IppsWTInvState_32f*)ippsMalloc_8u(pStateSize);
    check_sts(status = ippsWTInvInit_32f(pStateInv, invFltL, invFltLenL, invFltOffsL, invFltH, invFltLenH, invFltOffsH));
    /* For this particular case (non-shifted reconstruction)
     here is first data itself,
     that we need to place to delay line
     [(invFltLenL + invFltOffsL - 1) / 2] elements for l. filtering
     [(invFltLenH + invFltOffsH - 1) / 2] elements for h. filtering 
    */
    dLineInvL[0] = low[4]; dLineInvL[1] = low[5];    /* wrap [fwdFltLenL + fwdFltOffsL - 1] */
    dLineInvH[0] = high[4]; dLineInvH[1] =  high[5]; /* wrap [fwdFltLenL + fwdFltOffsL - 1] */

    check_sts(status = ippsWTInvSetDlyLine_32f(pStateInv, dLineInvL, dLineInvH));
    /* Inverse transform */
    check_sts(status = ippsWTInv_32f(&low[0], &high[0], 6, dst, pStateInv));
    /* print reconstruction result */
    printf("reconstruction:\n");
    for (i = 0; i < 12; i++) printf("%.0f; ", dst[i]);
    printf("\n");
    EXIT_MAIN
    ippsFree(pStateFwd);
    ippsFree(pStateInv);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}