/*******************************************************************************
* Copyright 2015-2021 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

// A simple example of forward/inverse wavelet  Daubechies 3 (db3) transform
// implemented with Intel(R) Integrated Primitives (Intel(R) IPP) functions:
//     ippiWTFwdGetSize_32f
//     ippiWTFwdInit_32f_C1R
//     ippiWTFwd_32f_C1R
//     ippiWTInvGetSize_32f
//     ippiWTInvInit_32f_C1R
//     ippiWTInv_32f_C1R

#include <stdio.h> 
#include "ipp.h"

#define FILTER_LENGTH 6
#define ANCHOR (FILTER_LENGTH-1)
#define EXIT_MAIN exitLine:                                  /* Label for Exit */
#define check_sts(st) if((st) != ippStsNoErr) goto exitLine; /* Go to Exit if Intel(R) IPP function returned status different from ippStsNoErr */

#define FwdPadL  0           //Pad size on Left:  taplen-1-anchorLow;
#define FwdPadR  5           //Pad size on Right:  anchor size
#define FwdPadT  0           //Pad size on Top:   TopBorder = LeftBorder
#define FwdPadB  5           //Pad size on Bottom: BottomBorder= rightBorder

#define InvPadL  2           //Pad size on Left: (lenLow - 1 - anchor) / 2;
#define InvPadR  0           //Pad size on Right: (anchor + 1)/2  
#define InvPadT  2           //Pad size on Top:  TopBorder = LeftBorder
#define InvPadB  0           //Pad size on Bottom: BottomBorder= rightBorder   

IppStatus status = ippStsNoErr;
static const Ipp32f pTapsLowF[FILTER_LENGTH] =
{/* Pointer to lowpass filter taps */
    0.035226291882100656f,
    -0.08544127388224149f,
    -0.13501102001039084f,
    0.4598775021193313f,
    0.8068915093133388f,
    0.3326705529509569f
};
static const Ipp32f pTapsHighF[FILTER_LENGTH] =
{/* Pointer to highpass filter taps */
    -0.3326705529509569f,
    0.8068915093133388f,
    -0.4598775021193313f,
    -0.13501102001039084f,
    0.08544127388224149f,
    0.035226291882100656f
};

static const Ipp32f pTapsLowI[FILTER_LENGTH] =
{/* Pointer to lowpass filter taps */
    0.3326705529509569f,
    0.8068915093133388f,
    0.4598775021193313f,
    -0.13501102001039084f,
    -0.08544127388224149f,
    0.035226291882100656f,
};
Ipp32f pTapsHighI[FILTER_LENGTH] =
{/* Pointer to highpass filter taps */
    0.035226291882100656f,
    0.08544127388224149f,
    -0.13501102001039084f,
    -0.4598775021193313f,
    0.8068915093133388f,
    -0.3326705529509569f
};

Ipp32f pSrc[8 * 8] =
{ /* the source image */
    1.1f, 0.0f, 0.0f, 11.1f, 11.1f, 0.0f, 0.0f, 1.2f,
    0.0f, 0.0f, 0.0f, 11.1f, 11.1f, 0.0f, 0.0f, 0.0f,
    0.0f, 0.0f, 0.0f, 11.1f, 11.1f, 0.0f, 0.0f, 0.0f,
    11.1f, 11.2f, 11.3f, 11.4f, 11.5f, 11.6f, 11.7f, 11.8f,
    11.1f, 11.1f, 11.1f, 11.1f, 11.1f, 11.1f, 11.1f, 11.1f,
    0.0f, 0.0f, 0.0f, 11.1f, 11.1f, 0.0f, 0.0f, 0.0f,
    0.0f, 0.0f, 0.0f, 11.1f, 11.1f, 0.0f, 0.0f, 0.0f,
    1.3f, 0.0f, 0.0f, 11.1f, 11.1f, 0.0f, 0.0f, 1.4f };

int main(void)
{
    int         i, j;
    int         lenHighF = FILTER_LENGTH, lenHighI = FILTER_LENGTH, lenLowF = FILTER_LENGTH, lenLowI = FILTER_LENGTH; /* Length of the lowpass/highpass filter */
    int         anchorHighF = ANCHOR, anchorHighI = 0, anchorLowF = ANCHOR, anchorLowI = 0;/* Anchor position of the lowpass/highpass filter */
    IppiWTFwdSpec_32f_C1R* pSpecFwd = NULL; /* Pointer to pointer to allocated ippiWTFwd spec structure */
    IppiWTInvSpec_32f_C1R* pSpecInv = NULL; /* Pointer to pointer to allocated ippiWTInv spec structure */
    int         pSpecSize;
    Ipp32f      pSrcB[(8 + FwdPadL + FwdPadR)*(8 + FwdPadT + FwdPadB)]; /* Pointer to source image ROI */
    int         srcStepB = (8 + FwdPadL + FwdPadR)*sizeof(Ipp32f);
    IppiSize    roiSizeB = { (8 + FwdPadL + FwdPadR), (8 + FwdPadT + FwdPadB) };
    int         srcStep = 8 * sizeof(Ipp32f);
    IppiSize    roiSize = { 8, 8 };
    Ipp32f      pDetailXDst[4 * 4]; /* Pointer to the destination "horizontal details" image ROI */
    Ipp32f      pDetailYDst[4 * 4]; /* Pointer to the destination "vertical details"   image ROI */
    Ipp32f      pDetailXYDst[4 * 4];/* Pointer to the destination "diagonal details"   image ROI */
    Ipp32f      pApproxDst[4 * 4];  /* Pointer to destination "approximation" image ROI */
    IppiSize    dstRoiSize = { 4, 4 }; /* ROI size for all destination images */
    int         pBufSize;
    Ipp8u*      pBuffer = NULL;
    Ipp32f      pDstInv[8 * 8]; /* Pointer to the destination image ROI */
    int         stepDstInv = 8 * sizeof(Ipp32f);
    int         numChannels = 1;
    Ipp32f      pAppB[(4 + InvPadL + InvPadR)*(4 + InvPadT + InvPadB)];/* Pointer to the source "approximation" image ROI */
    Ipp32f      pXB[(4 + InvPadL + InvPadR)*(4 + InvPadT + InvPadB)];
    Ipp32f      pYB[(4 + InvPadL + InvPadR)*(4 + InvPadT + InvPadB)];
    Ipp32f      pXYB[(4 + InvPadL + InvPadR)*(4 + InvPadT + InvPadB)];
    int         StepB = (4 + InvPadL + InvPadR)*sizeof(Ipp32f);
    IppiSize    roiInvSize = { 4, 4 };
    IppiSize    roiInvSizeB = { 4 + InvPadL + InvPadR, 4 + InvPadT + InvPadB };
    int         approxStep, detailXStep, detailYStep, detailXYStep;

    approxStep = detailXStep = detailYStep = detailXYStep = 4 * sizeof(Ipp32f);
    /* adds border to the source image */
    check_sts(status = ippiCopyWrapBorder_32s_C1R((Ipp32s*)pSrc, srcStep, roiSize, (Ipp32s*)pSrcB, srcStepB, roiSizeB, FwdPadT, FwdPadL));
    /* performs forward  wavelet transform */
    check_sts(status = ippiWTFwdGetSize_32f(numChannels, lenLowF, anchorLowF, lenHighF, anchorHighF, &pSpecSize, &pBufSize));
    pBuffer = ippsMalloc_8u(pBufSize);
    pSpecFwd = (IppiWTFwdSpec_32f_C1R*)ippsMalloc_8u(pSpecSize);
    check_sts(status = ippiWTFwdInit_32f_C1R(pSpecFwd, pTapsLowF, lenLowF, anchorLowF, pTapsHighF, lenHighF, anchorHighF));
    check_sts(status = ippiWTFwd_32f_C1R(pSrcB + FwdPadT*roiSizeB.width + FwdPadL, srcStepB, pApproxDst, approxStep, pDetailXDst, 
                                         detailXStep, pDetailYDst, detailYStep, pDetailXYDst, detailXYStep, dstRoiSize, pSpecFwd, pBuffer));

    ippsFree(pBuffer);
    check_sts(status = ippiWTInvGetSize_32f(numChannels, lenLowI, anchorLowI, lenHighI, anchorHighI, &pSpecSize, &pBufSize));
    pBuffer = ippsMalloc_8u(pBufSize);
    pSpecInv = (IppiWTInvSpec_32f_C1R*)ippsMalloc_8u(pSpecSize);
    check_sts(status = ippiWTInvInit_32f_C1R(pSpecInv, pTapsLowI, lenLowI, anchorLowI, pTapsHighI, lenHighI, anchorHighI));
    /* adds border to four images obtained after ippiWTFwd */
    check_sts(status = ippiCopyWrapBorder_32s_C1R((Ipp32s*)pApproxDst, approxStep, dstRoiSize, (Ipp32s*)pAppB, StepB, roiInvSizeB, InvPadT, InvPadL));
    check_sts(status = ippiCopyWrapBorder_32s_C1R((Ipp32s*)pDetailXDst, detailXStep, dstRoiSize, (Ipp32s*)pXB, StepB, roiInvSizeB, InvPadT, InvPadL));
    check_sts(status = ippiCopyWrapBorder_32s_C1R((Ipp32s*)pDetailYDst, detailYStep, dstRoiSize, (Ipp32s*)pYB, StepB, roiInvSizeB, InvPadT, InvPadL));
    check_sts(status = ippiCopyWrapBorder_32s_C1R((Ipp32s*)pDetailXYDst, detailXYStep, dstRoiSize, (Ipp32s*)pXYB, StepB, roiInvSizeB, InvPadT, InvPadL));

    /* performs inverse  wavelet transform */
    check_sts(status = ippiWTInv_32f_C1R(pAppB + InvPadT*roiInvSizeB.width + InvPadL, StepB, pXB + InvPadT*roiInvSizeB.width + InvPadL, StepB, 
                                         pYB + InvPadT*roiInvSizeB.width + InvPadL, StepB, pXYB + InvPadT*roiInvSizeB.width + InvPadL, StepB, 
                                         roiInvSize, pDstInv, stepDstInv, pSpecInv, pBuffer));

    for ( i = 0; i<8; i++){
        for ( j = 0; j<8; j++) printf("%8.3f ", pDstInv[i * 8 + j]);
        printf("\n");
    }
EXIT_MAIN
    ippsFree(pSpecFwd);
    ippsFree(pBuffer);
    ippsFree(pSpecInv);
    printf("Exit status %d (%s)\n", (int)status, ippGetStatusString(status));
    return (int)status;
}